#pragma once

#define WIN32_LEAN_AND_MEAN     // Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <stdio.h>

#define ITEM    0
#define CRITTER 1
#define SCENERY 2
#define WALL    3
#define TILE    4
#define MISC    5

char* lpszObjectType[] = {
    "Item",
    "Critter",
    "Scenery",
    "Wall",
    "Tile",
    "Misc",
    "Unknown!!!"
};

#define ARMOR       0
#define CONTAINER   1
#define DRUG        2
#define WEAPON      3
#define AMMO        4
#define MISCITEM    5
#define KEY         6

char* lpszItemType[] = {
    "Armor",
    "Container",
    "Drug",
    "Weapon",
    "Ammo",
    "Misc Item",
    "Key",
    "Unknown!!!"
};

#define PORTAL          0
#define STAIRS          1
#define ELEVATORS       2
#define LADDERBOTTOM    3
#define LADDERTOP       4
#define GENERICSCENERY  5

char* lpszSceneryType[] = {
    "Portal",
    "Stairs",
    "Elevators",
    "Ladder Bottom",
    "Ladder Top",
    "Generic Scenery",
    "Unknown!!!"
};

void ReverseWord(WORD& w)
{
    w = (w >> 8) | (w << 8);
}

void ReverseDWord(DWORD& dw)
{
    dw = (dw >> 24) | (dw << 24) | ((dw >> 8) & 0x0000FF00) | ((dw << 8) & 0x00FF0000);
}

int main(int argc, char* argv[])
{
    if (argc == 1) {
        printf("Usage: %s file.pro\n", argv[0]);
        return 1;
    }

    HANDLE hPro = CreateFile(argv[1],
                             GENERIC_READ,
                             FILE_SHARE_READ,
                             NULL,
                             OPEN_EXISTING,
                             FILE_ATTRIBUTE_NORMAL,
                             NULL);

    if (hPro == INVALID_HANDLE_VALUE) {
        printf("Error: Unable open %s\n", argv[1]);
        return 1;
    }

    printf("File: %s\n", argv[1]);
    printf("\n");
    printf("Common fields\n");

    DWORD dwRead;
    DWORD dwValue;
    BYTE  nValue;
    DWORD offset = 0;
    DWORD i;
    BYTE  nObjectType;
    DWORD dwObjectID;

    // Pro ID
    ReadFile(hPro,
             &dwValue,
             4,
             &dwRead,
             NULL);
    ReverseDWord(dwValue);
    nObjectType = BYTE(dwValue >> 24);
    nObjectType = (nObjectType > 5) ? 6 : nObjectType;
    dwObjectID = dwValue & 0x00FFFFFF;
    printf("0x%08x: %08x\t\t\t\t<= Proto ID (%d), ObjectType: %2d - %s, ObjectID: %d\n",
           offset,
           dwValue,
           dwValue,
           nObjectType,
           lpszObjectType[nObjectType],
           dwObjectID);
    offset += 4;

    // Description ID
    ReadFile(hPro,
             &dwValue,
             4,
             &dwRead,
             NULL);
    ReverseDWord(dwValue);
    printf("0x%08x: %08x\t\t\t\t<= Description ID (%d)\n",
           offset,
           dwValue,
           dwValue);
    offset += 4;

    // Item FRMID
    ReadFile(hPro,
             &dwValue,
             4,
             &dwRead,
             NULL);
    ReverseDWord(dwValue);
    printf("0x%08x: %08x\t\t\t\t<= FRM ID (%d), ID - %d\n",
           offset,
           dwValue,
           dwValue,
           dwValue & 0x00000FFF);
    offset += 4;

    switch(nObjectType) {
        case ITEM:
            printf("\n");
            printf("Item common fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x0c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x10 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Flags
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Flags (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x18 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x1c
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x1c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Item Type
            DWORD dwItemType;

            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            dwItemType = dwValue;
            dwItemType = (dwItemType > 6) ? 7 : dwItemType;
            printf("0x%08x: %08x\t\t\t\t<= Item Type (%d - %s)\n",
                   offset,
                   dwItemType,
                   dwItemType,
                   lpszItemType[dwItemType]);
            offset += 4;

            // Material ID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Material ID (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x28
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x28 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Weight
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Weight (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Price
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Price (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Inventory FRMID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Inventory FRMID (%d)\n",
                   offset,
                   dwValue,
                   dwValue & 0x00000FFF);
            offset += 4;

            // 0x38
            ReadFile(hPro,
                     &nValue,
                     1,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %02x\t\t\t\t\t<= Field_0x38 (%d)\n",
                   offset,
                   nValue,
                   nValue);
            offset += 1;

            switch(dwItemType) {
                case ARMOR:
                    printf("\n");
                    printf("Armor fields\n");

                    // Armor class
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Armor class (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Damage resistance
                    DWORD DR[7];

                    ReadFile(hPro,
                             DR,
                             7 * 4,
                             &dwRead,
                             NULL);

                    for(i = 0; i < 7; i++)
                        ReverseDWord(DR[i]);

                    printf("0x%08x:",
                            offset);
                    printf(" %08x %08x %08x %08x",
                            DR[0],
                            DR[1],
                            DR[2],
                            DR[3]);
                    printf("\t<= Damage resistance (Normal: %d, Laser: %d, Fire: %d, Plasma: %d)\n",
                            DR[0],
                            DR[1],
                            DR[2],
                            DR[3]);
                    printf("\t    %08x %08x %08x",
                            DR[4],
                            DR[5],
                            DR[6]);
                    printf("\t\t\t\t    (Electrical: %d, EMP: %d, Explosion: %d)\n",
                            DR[4],
                            DR[5],
                            DR[6]);
                    offset += (7 * 4);

                    // Damage treshold
                    DWORD DT[7];

                    ReadFile(hPro,
                             DT,
                             7 * 4,
                             &dwRead,
                             NULL);

                    for(i = 0; i < 7; i++)
                        ReverseDWord(DT[i]);

                    printf("0x%08x:",
                            offset);
                    printf(" %08x %08x %08x %08x",
                            DT[0],
                            DT[1],
                            DT[2],
                            DT[3]);
                    printf("\t<= Damage threshold (Normal: %d, Laser: %d, Fire: %d, Plasma: %d)\n",
                            DT[0],
                            DT[1],
                            DT[2],
                            DT[3]);
                    printf("\t    %08x %08x %08x",
                            DT[4],
                            DT[5],
                            DT[6]);
                    printf("\t\t\t\t     (Electrical: %d, EMP: %d, Explosion: %d)\n",
                            DT[4],
                            DT[5],
                            DT[6]);
                    offset += (7 * 4);

                    // Perk ID
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Perk ID (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Player Female FRMID
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Player Female FRMID (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Player Female FRMID
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Player Male FRMID (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case CONTAINER:
                    printf("\n");
                    printf("Container fields\n");

                    // Capacity
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Capacity (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x3d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x3d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case DRUG:
                    printf("\n");
                    printf("Drug fields\n");

                    // StatID0
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= StatID0 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // StatID1
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= StatID1 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // StatID2
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= StatID2 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Immediate stat modifiers
                    DWORD SM[7];

                    ReadFile(hPro,
                             SM,
                             3 * 4,
                             &dwRead,
                             NULL);

                    for(i = 0; i < 3; i++)
                        ReverseDWord(SM[i]);

                    printf("0x%08x: ",
                            offset);
                    printf("%08x %08x %08x",
                            SM[0],
                            SM[1],
                            SM[2]);
                    printf("\t\t<= Immediate stat modifiers (Stat1: %d, Stat2: %d, Stat3: %d)\n",
                            SM[0],
                            SM[1],
                            SM[2]);
                    offset += (3 * 4);

                    // Intermediate effect delay
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Intermediate effect delay (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Intermediate stat modifiers
                    ReadFile(hPro,
                             SM,
                             3 * 4,
                             &dwRead,
                             NULL);

                    for(i = 0; i < 3; i++)
                        ReverseDWord(SM[i]);

                    printf("0x%08x: ",
                            offset);
                    printf("%08x %08x %08x",
                            SM[0],
                            SM[1],
                            SM[2]);
                    printf("\t\t<= Intermediate stat modifiers (Stat1: %d, Stat2: %d, Stat3: %d)\n",
                            SM[0],
                            SM[1],
                            SM[2]);
                    offset += (3 * 4);

                    // End effect delay
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= End effect time (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // End stat modifiers
                    printf("0x%08x: ",
                            offset);
                    ReadFile(hPro,
                             SM,
                             3 * 4,
                             &dwRead,
                             NULL);

                    for(i = 0; i < 3; i++)
                        ReverseDWord(SM[i]);

                    printf("%08x %08x %08x",
                            SM[0],
                            SM[1],
                            SM[2]);
                    printf("\t\t<= End stat modifiers (Stat1: %d, Stat2: %d, Stat3: %d)\n",
                            SM[0],
                            SM[1],
                            SM[2]);
                    offset += (3 * 4);

                    // Addiction rate
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Addiction rate (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Addiction PerkID
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Addiction PerkID (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // Addiction Delay
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Addiction Delay (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case WEAPON:
                    printf("\n");
                    printf("Weapon fields\n");

                    // 0x39
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x39 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x3d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x3d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x41
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x41 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x45
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x45 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x49
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x49 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x4d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x4d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x51
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x51 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x55
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x55 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x59
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x59 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x5d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x5d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x61
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x61 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x65
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x65 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x69
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x69 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x6d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x6d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x71
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x71 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x75
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x75 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x79
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x79 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case AMMO:
                    printf("\n");
                    printf("Ammo fields\n");

                    // 0x39
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x39 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x3d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x3d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x41
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x41 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x45
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x45 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x49
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x49 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x4d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x4d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case MISCITEM:
                    printf("\n");
                    printf("Misk item fields\n");

                    // 0x39
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x39 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x3d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x3d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x41
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x41 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case KEY:
                    printf("\n");
                    printf("Key fields\n");

                    // 0x39
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x39 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                default:
                    printf("Unknown item type!!!\n");
            };

            break;

        case CRITTER:
            printf("\n");
            printf("Critter fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Light distance (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Light intensity (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x14
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Flags (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Flags Ext (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x1C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Script ID (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x20
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Head FRM ID (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x24
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= AI Packet (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x28
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Team Num (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x2C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Critter Flags (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x30-0xbb
            DWORD DWA[35];
            ReadFile(hPro,
                     DWA,
                     4 * 35,
                     &dwRead,
                     NULL);
            for(i = 0; i < 35; i++)
                ReverseDWord(DWA[i]);

            printf("\n");
            printf("Characteristics\n");
            //printf("0x%08x:\n",
            //       offset);
            printf("0x%08x: %08x\t\t\t\t<= Strength (%d)\n",
                   offset,
                   DWA[0],
                   DWA[0]);
            printf("0x%08x: %08x\t\t\t\t<= Perception (%d)\n",
                   offset + 4,
                   DWA[1],
                   DWA[1]);
            printf("0x%08x: %08x\t\t\t\t<= Endurance (%d)\n",
                   offset + 8,
                   DWA[2],
                   DWA[2]);
            printf("0x%08x: %08x\t\t\t\t<= Charisma (%d)\n",
                   offset + 12,
                   DWA[3],
                   DWA[3]);
            printf("0x%08x: %08x\t\t\t\t<= Intelligence (%d)\n",
                   offset + 16,
                   DWA[4],
                   DWA[4]);
            printf("0x%08x: %08x\t\t\t\t<= Agility (%d)\n",
                   offset + 20,
                   DWA[5],
                   DWA[5]);
            printf("0x%08x: %08x\t\t\t\t<= Luck (%d)\n",
                   offset + 24,
                   DWA[6],
                   DWA[6]);
            printf("0x%08x: %08x\t\t\t\t<= HP (%d)\n",
                   offset + 28,
                   DWA[7],
                   DWA[7]);
            printf("0x%08x: %08x\t\t\t\t<= AP (%d)\n",
                   offset + 32,
                   DWA[8],
                   DWA[8]);
            printf("0x%08x: %08x\t\t\t\t<= AC (%d)\n",
                   offset + 36,
                   DWA[9],
                   DWA[9]);
            printf("0x%08x: %08x\t\t\t\t<= Unarmed damage (%d)\n",
                   offset + 40,
                   DWA[10],
                   DWA[10]);
            printf("0x%08x: %08x\t\t\t\t<= Melee damage (%d)\n",
                   offset + 44,
                   DWA[11],
                   DWA[11]);
            printf("0x%08x: %08x\t\t\t\t<= Carry weight (%d)\n",
                   offset + 48,
                   DWA[12],
                   DWA[12]);
            printf("0x%08x: %08x\t\t\t\t<= Sequence (%d)\n",
                   offset + 52,
                   DWA[13],
                   DWA[13]);
            printf("0x%08x: %08x\t\t\t\t<= Healing rate (%d)\n",
                   offset + 56,
                   DWA[14],
                   DWA[14]);
            printf("0x%08x: %08x\t\t\t\t<= Critical chance (%d)\n",
                   offset + 60,
                   DWA[15],
                   DWA[15]);
            printf("0x%08x: %08x\t\t\t\t<= Better criticals (%d)\n",
                   offset + 64,
                   DWA[16],
                   DWA[16]);
            printf("0x%08x: %08x\t\t\t\t<= DT Normal (%d)\n",
                   offset + 68,
                   DWA[17],
                   DWA[17]);
            printf("0x%08x: %08x\t\t\t\t<= DT Laser (%d)\n",
                   offset + 72,
                   DWA[18],
                   DWA[18]);
            printf("0x%08x: %08x\t\t\t\t<= DT Fire (%d)\n",
                   offset + 76,
                   DWA[19],
                   DWA[19]);
            printf("0x%08x: %08x\t\t\t\t<= DT Plasma (%d)\n",
                   offset + 80,
                   DWA[20],
                   DWA[20]);
            printf("0x%08x: %08x\t\t\t\t<= DT Electrical (%d)\n",
                   offset + 84,
                   DWA[21],
                   DWA[21]);
            printf("0x%08x: %08x\t\t\t\t<= DT EMP (%d)\n",
                   offset + 88,
                   DWA[22],
                   DWA[22]);
            printf("0x%08x: %08x\t\t\t\t<= DT Explode (%d)\n",
                   offset + 92,
                   DWA[23],
                   DWA[23]);
            printf("0x%08x: %08x\t\t\t\t<= DR Normal (%d)\n",
                   offset + 96,
                   DWA[24],
                   DWA[24]);
            printf("0x%08x: %08x\t\t\t\t<= DR Laser (%d)\n",
                   offset + 100,
                   DWA[25],
                   DWA[25]);
            printf("0x%08x: %08x\t\t\t\t<= DR Fire (%d)\n",
                   offset + 104,
                   DWA[26],
                   DWA[26]);
            printf("0x%08x: %08x\t\t\t\t<= DR Plasma (%d)\n",
                   offset + 108,
                   DWA[27],
                   DWA[27]);
            printf("0x%08x: %08x\t\t\t\t<= DR Electrical (%d)\n",
                   offset + 112,
                   DWA[28],
                   DWA[28]);
            printf("0x%08x: %08x\t\t\t\t<= DR EMP (%d)\n",
                   offset + 116,
                   DWA[29],
                   DWA[29]);
            printf("0x%08x: %08x\t\t\t\t<= DR Explode (%d)\n",
                   offset + 120,
                   DWA[30],
                   DWA[30]);
            printf("0x%08x: %08x\t\t\t\t<= DR Radiation (%d)\n",
                   offset + 124,
                   DWA[31],
                   DWA[31]);
            printf("0x%08x: %08x\t\t\t\t<= DR Poison (%d)\n",
                   offset + 128,
                   DWA[32],
                   DWA[32]);
            printf("0x%08x: %08x\t\t\t\t<= Age (%d)\n",
                   offset + 132,
                   DWA[33],
                   DWA[33]);
            printf("0x%08x: %08x\t\t\t\t<= Gender (0 - male, 1 - female) (%d)\n",
                   offset + 136,
                   DWA[34],
                   DWA[34]);
            offset += (4 * 35);

            // 0xbc-0x147
            ReadFile(hPro,
                     DWA,
                     4 * 35,
                     &dwRead,
                     NULL);
            for(i = 0; i < 35; i++)
                ReverseDWord(DWA[i]);

            printf("\n");
            printf("Characteristics bonuses\n");
            //printf("0x%08x:\n",
            //       offset);
            printf("0x%08x: %08x\t\t\t\t<= Strength (%d)\n",
                   offset,
                   DWA[0],
                   DWA[0]);
            printf("0x%08x: %08x\t\t\t\t<= Perception (%d)\n",
                   offset + 4,
                   DWA[1],
                   DWA[1]);
            printf("0x%08x: %08x\t\t\t\t<= Endurance (%d)\n",
                   offset + 8,
                   DWA[2],
                   DWA[2]);
            printf("0x%08x: %08x\t\t\t\t<= Charisma (%d)\n",
                   offset + 12,
                   DWA[3],
                   DWA[3]);
            printf("0x%08x: %08x\t\t\t\t<= Intelligence (%d)\n",
                   offset + 16,
                   DWA[4],
                   DWA[4]);
            printf("0x%08x: %08x\t\t\t\t<= Agility (%d)\n",
                   offset + 20,
                   DWA[5],
                   DWA[5]);
            printf("0x%08x: %08x\t\t\t\t<= Luck (%d)\n",
                   offset + 24,
                   DWA[6],
                   DWA[6]);
            printf("0x%08x: %08x\t\t\t\t<= HP (%d)\n",
                   offset + 28,
                   DWA[7],
                   DWA[7]);
            printf("0x%08x: %08x\t\t\t\t<= AP (%d)\n",
                   offset + 32,
                   DWA[8],
                   DWA[8]);
            printf("0x%08x: %08x\t\t\t\t<= AC (%d)\n",
                   offset + 36,
                   DWA[9],
                   DWA[9]);
            printf("0x%08x: %08x\t\t\t\t<= Unarmed damage (%d)\n",
                   offset + 40,
                   DWA[10],
                   DWA[10]);
            printf("0x%08x: %08x\t\t\t\t<= Melee damage (%d)\n",
                   offset + 44,
                   DWA[11],
                   DWA[11]);
            printf("0x%08x: %08x\t\t\t\t<= Carry weight (%d)\n",
                   offset + 48,
                   DWA[12],
                   DWA[12]);
            printf("0x%08x: %08x\t\t\t\t<= Sequence (%d)\n",
                   offset + 52,
                   DWA[13],
                   DWA[13]);
            printf("0x%08x: %08x\t\t\t\t<= Healing rate (%d)\n",
                   offset + 56,
                   DWA[14],
                   DWA[14]);
            printf("0x%08x: %08x\t\t\t\t<= Critical chance (%d)\n",
                   offset + 60,
                   DWA[15],
                   DWA[15]);
            printf("0x%08x: %08x\t\t\t\t<= Better criticals (%d)\n",
                   offset + 64,
                   DWA[16],
                   DWA[16]);
            printf("0x%08x: %08x\t\t\t\t<= DT Normal (%d)\n",
                   offset + 68,
                   DWA[17],
                   DWA[17]);
            printf("0x%08x: %08x\t\t\t\t<= DT Laser (%d)\n",
                   offset + 72,
                   DWA[18],
                   DWA[18]);
            printf("0x%08x: %08x\t\t\t\t<= DT Fire (%d)\n",
                   offset + 76,
                   DWA[19],
                   DWA[19]);
            printf("0x%08x: %08x\t\t\t\t<= DT Plasma (%d)\n",
                   offset + 80,
                   DWA[20],
                   DWA[20]);
            printf("0x%08x: %08x\t\t\t\t<= DT Electrical (%d)\n",
                   offset + 84,
                   DWA[21],
                   DWA[21]);
            printf("0x%08x: %08x\t\t\t\t<= DT EMP (%d)\n",
                   offset + 88,
                   DWA[22],
                   DWA[22]);
            printf("0x%08x: %08x\t\t\t\t<= DT Explode (%d)\n",
                   offset + 92,
                   DWA[23],
                   DWA[23]);
            printf("0x%08x: %08x\t\t\t\t<= DR Normal (%d)\n",
                   offset + 96,
                   DWA[24],
                   DWA[24]);
            printf("0x%08x: %08x\t\t\t\t<= DR Laser (%d)\n",
                   offset + 100,
                   DWA[25],
                   DWA[25]);
            printf("0x%08x: %08x\t\t\t\t<= DR Fire (%d)\n",
                   offset + 104,
                   DWA[26],
                   DWA[26]);
            printf("0x%08x: %08x\t\t\t\t<= DR Plasma (%d)\n",
                   offset + 108,
                   DWA[27],
                   DWA[27]);
            printf("0x%08x: %08x\t\t\t\t<= DR Electrical (%d)\n",
                   offset + 112,
                   DWA[28],
                   DWA[28]);
            printf("0x%08x: %08x\t\t\t\t<= DR EMP (%d)\n",
                   offset + 116,
                   DWA[29],
                   DWA[29]);
            printf("0x%08x: %08x\t\t\t\t<= DR Explode (%d)\n",
                   offset + 120,
                   DWA[30],
                   DWA[30]);
            printf("0x%08x: %08x\t\t\t\t<= DR Radiation (%d)\n",
                   offset + 124,
                   DWA[31],
                   DWA[31]);
            printf("0x%08x: %08x\t\t\t\t<= DR Poison (%d)\n",
                   offset + 128,
                   DWA[32],
                   DWA[32]);
            printf("0x%08x: %08x\t\t\t\t<= Age (%d)\n",
                   offset + 132,
                   DWA[33],
                   DWA[33]);
            printf("0x%08x: %08x\t\t\t\t<= Gender (0 - male, 1 - female) (%d)\n",
                   offset + 136,
                   DWA[34],
                   DWA[34]);
            offset += (4 * 35);

            // 0x148-0x18f
            ReadFile(hPro,
                     DWA,
                     4 * 18,
                     &dwRead,
                     NULL);
            for(i = 0; i < 18; i++)
                ReverseDWord(DWA[i]);

            printf("\n");
            printf("Skills\n");
            //printf("0x%08x:\n",
            //       offset);
            printf("0x%08x: %08x\t\t\t\t<= Small guns (%d)\n",
                   offset,
                   DWA[0],
                   DWA[0]);
            printf("0x%08x: %08x\t\t\t\t<= Big guns (%d)\n",
                   offset + 4,
                   DWA[1],
                   DWA[1]);
            printf("0x%08x: %08x\t\t\t\t<= Energy weapons (%d)\n",
                   offset + 8,
                   DWA[2],
                   DWA[2]);
            printf("0x%08x: %08x\t\t\t\t<= Unarmed (%d)\n",
                   offset + 12,
                   DWA[3],
                   DWA[3]);
            printf("0x%08x: %08x\t\t\t\t<= Melee (%d)\n",
                   offset + 16,
                   DWA[4],
                   DWA[4]);
            printf("0x%08x: %08x\t\t\t\t<= Throwing (%d)\n",
                   offset + 20,
                   DWA[5],
                   DWA[5]);
            printf("0x%08x: %08x\t\t\t\t<= First aid (%d)\n",
                   offset + 24,
                   DWA[6],
                   DWA[6]);
            printf("0x%08x: %08x\t\t\t\t<= Doctor (%d)\n",
                   offset + 28,
                   DWA[7],
                   DWA[7]);
            printf("0x%08x: %08x\t\t\t\t<= Sneak (%d)\n",
                   offset + 32,
                   DWA[8],
                   DWA[8]);
            printf("0x%08x: %08x\t\t\t\t<= Lockpick (%d)\n",
                   offset + 36,
                   DWA[9],
                   DWA[9]);
            printf("0x%08x: %08x\t\t\t\t<= Steal (%d)\n",
                   offset + 40,
                   DWA[10],
                   DWA[10]);
            printf("0x%08x: %08x\t\t\t\t<= Traps (%d)\n",
                   offset + 44,
                   DWA[11],
                   DWA[11]);
            printf("0x%08x: %08x\t\t\t\t<= Science (%d)\n",
                   offset + 48,
                   DWA[12],
                   DWA[12]);
            printf("0x%08x: %08x\t\t\t\t<= Repair (%d)\n",
                   offset + 52,
                   DWA[13],
                   DWA[13]);
            printf("0x%08x: %08x\t\t\t\t<= Speech (%d)\n",
                   offset + 56,
                   DWA[14],
                   DWA[14]);
            printf("0x%08x: %08x\t\t\t\t<= Barter (%d)\n",
                   offset + 60,
                   DWA[15],
                   DWA[15]);
            printf("0x%08x: %08x\t\t\t\t<= Gambling (%d)\n",
                   offset + 64,
                   DWA[16],
                   DWA[16]);
            printf("0x%08x: %08x\t\t\t\t<= Outdoorsman (%d)\n",
                   offset + 68,
                   DWA[17],
                   DWA[17]);
            offset += (4 * 18);

            // Body Type ID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Body Type (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Expirience points
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Exp Val (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Kill type name ID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Kill Type (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x19C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Damage Type (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;
            break;

        case SCENERY:
            printf("\n");
            printf("Scenery common fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x0c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x10 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x14
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x14 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x18 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x1c
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x1c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x20
            DWORD dwSceneryType;

            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            dwSceneryType = dwValue;
            dwSceneryType = (dwSceneryType > 5) ? 6 : dwSceneryType;
            printf("0x%08x: %08x\t\t\t\t<= Scenery Type (%d - %s)\n",
                   offset,
                   dwSceneryType,
                   dwSceneryType,
                   lpszSceneryType[dwSceneryType]);
            offset += 4;

            // Material ID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Material ID (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x28
            ReadFile(hPro,
                     &nValue,
                     1,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %02x\t\t\t\t\t<= Field_0x28 (%d)\n",
                   offset,
                   nValue,
                   nValue);
            offset += 1;

            switch(dwSceneryType) {
                case PORTAL:
                    printf("\n");
                    printf("Portal fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x2d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x2d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case STAIRS:
                    printf("\n");
                    printf("Stairs fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x2d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x2d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case ELEVATORS:
                    printf("\n");
                    printf("Elevator fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;

                    // 0x2d
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x2d (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case LADDERBOTTOM:
                    printf("\n");
                    printf("Ladder bottom fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case LADDERTOP:
                    printf("\n");
                    printf("Ladder top fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                case GENERICSCENERY:
                    printf("\n");
                    printf("Generic scenery fields\n");

                    // 0x29
                    ReadFile(hPro,
                             &dwValue,
                             4,
                             &dwRead,
                             NULL);
                    ReverseDWord(dwValue);
                    printf("0x%08x: %08x\t\t\t\t<= Field_0x29 (%d)\n",
                           offset,
                           dwValue,
                           dwValue);
                    offset += 4;
                    break;

                default:
                    printf("Unknow scenery type!!!");
            }

            break;

        case WALL:
            printf("\n");
            printf("Wall fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x0c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x10 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x14
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x14 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x18 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x1c
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x1c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // Material ID
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Material ID (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;
            break;

        case TILE:
            printf("\n");
            printf("Tiles fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x0c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x10 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x14
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x14 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x18 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;
            break;

        case MISC:
            printf("\n");
            printf("Misc object fields\n");

            // 0x0C
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x0c (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x10
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x10 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x14
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x14 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;

            // 0x18
            ReadFile(hPro,
                     &dwValue,
                     4,
                     &dwRead,
                     NULL);
            ReverseDWord(dwValue);
            printf("0x%08x: %08x\t\t\t\t<= Field_0x18 (%d)\n",
                   offset,
                   dwValue,
                   dwValue);
            offset += 4;
            break;

        default:
            printf("Unknown object type!!!\n");
    }

    printf("\n");

    CloseHandle(hPro);
    return 0;
}

